using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.IO;

namespace SmartmaniaGame
{
    public static class Operations
    {
        /// <summary>
        /// Cache of the blendstates needed for the LoadTextureFromStream method
        /// </summary>
        private static BlendState blendAlpha = null, blendColor = null;

        /// <summary>
        /// Load texture from file - without Content Pipeline (faster loading of large images!)
        /// - LoadTextureFromStream method to speed up loading Texture2Ds from pngs,
        ///   as described in http://jakepoz.com/jake_poznanski__speeding_up_xna.html
        /// - filename must be set with "Content/" and ".png" or ".jpg"
        /// </summary>
        /// <returns>A Texture2D with premultiplied alpha</returns>
        public static Texture2D LoadTextureFromStream(GraphicsDevice device, string fileName)
        {
            Texture2D file = null;
            RenderTarget2D result = null;

            using (Stream titleStream = TitleContainer.OpenStream(fileName))
            {
                file = Texture2D.FromStream(device, titleStream);
            }

            //Setup a render target to hold our final texture which will have premulitplied alpha values
            result = new RenderTarget2D(device, file.Width, file.Height);
            device.SetRenderTarget(result);
            device.Clear(Color.Black);

            //Multiply each color by the source alpha, and write in just the color values into the final texture
            if (blendColor == null)
            {
                blendColor = new BlendState();
                blendColor.ColorWriteChannels = ColorWriteChannels.Red | ColorWriteChannels.Green | ColorWriteChannels.Blue;
                blendColor.AlphaDestinationBlend = Blend.Zero;
                blendColor.ColorDestinationBlend = Blend.Zero;
                blendColor.AlphaSourceBlend = Blend.SourceAlpha;
                blendColor.ColorSourceBlend = Blend.SourceAlpha;
            }
            SpriteBatch spriteBatch = new SpriteBatch(device);
            spriteBatch.Begin(SpriteSortMode.Immediate, blendColor);
            spriteBatch.Draw(file, file.Bounds, Color.White);
            spriteBatch.End();

            //Now copy over the alpha values from the PNG source texture to the final one, without multiplying them
            if (blendAlpha == null)
            {
                blendAlpha = new BlendState();
                blendAlpha.ColorWriteChannels = ColorWriteChannels.Alpha;
                blendAlpha.AlphaDestinationBlend = Blend.Zero;
                blendAlpha.ColorDestinationBlend = Blend.Zero;
                blendAlpha.AlphaSourceBlend = Blend.One;
                blendAlpha.ColorSourceBlend = Blend.One;
            }
            spriteBatch.Begin(SpriteSortMode.Immediate, blendAlpha);
            spriteBatch.Draw(file, file.Bounds, Color.White);
            spriteBatch.End();

            //Release the GPU back to drawing to the screen
            device.SetRenderTarget(null);
            return result as Texture2D;
        }

    }
}
